﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.ColladaDom
{
    // .NET Fx
    using System;
    using System.Xml.Serialization;

    /// <summary>Describes the visual shape and appearance of an object in a scene.</summary>
    /// <remarks>
    ///     <para>The &lt;geometry&gt; element categorizes the declaration of geometric information.
    ///     Geometry is a branch of mathematics that deals with the measurement, properties, and
    ///     relationships of points, lines, angles, surfaces, and solids. The &lt;geometry&gt; element
    ///     contains a declaration of a mesh, convex mesh, or spline.</para>
    ///     <para>There are many forms of geometric description. Computer graphics hardware has been
    ///     normalized, primarily, to accept vertex position information with varying degrees of
    ///     attribution (color, normals, etc.). Geometric descriptions provide this vertex data with
    ///     relative directness or efficiency. Some of the more common forms of geometry are:
    ///         <list>
    ///             <item>B-Spline</item>
    ///             <item>Bézier</item>
    ///             <item>Mesh</item>
    ///             <item>NURBS</item>
    ///             <item>Patch</item>
    ///         </list>
    ///     </para>
    ///     <para>This is by no means an exhaustive list. Currently, COLLADA supports only polygonal
    ///     meshes and splines.</para>
    /// </remarks>
    public sealed class GeometryTag
    {
        /// <summary>A text string containing the unique identifier of the element. This value must
        /// be unique within the instance document.</summary>
        /// <remarks>Optional.</remarks>
        [XmlAttribute(AttributeName = "id", DataType = "ID")]
        public string Id { get; set; }

        /// <summary>The text string name of this element.</summary>
        /// <remarks>Optional.</remarks>
        [XmlAttribute(AttributeName = "name", DataType = "token")]
        public string Name { get; set; }

        /// <summary>Defines asset-management information regarding its parent element.</summary>
        [XmlElement(ElementName = "asset")]
        public AssetTag Asset { get; set; }

        /// <summary>The element that describes geometric data.</summary>
        [XmlChoiceIdentifier(MemberName = "GeometricElementType")]
        [XmlElement(ElementName = "convex_mesh", Type = typeof(Physics.ConvexMeshTag))]
        [XmlElement(ElementName = "mesh", Type = typeof(MeshTag))]
        [XmlElement(ElementName = "spline", Type = typeof(SplineTag))]
        // TODO: [XmlElement(ElementName = "brep", Type = typeof(BoundaryRep.BoundaryRepTag))]
        public object GeometricElement { get; set; }

        [XmlIgnore]
        public GeometricElementType GeometricElementType { get; set; }

        /// <summary>Provides arbitrary additional information about or related to its parent element.</summary>
        [XmlElement(ElementName = "extra")]
        public ExtraTag[] Extras { get; set; }
    }
}
