﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.ColladaDom.Physics
{
    // .NET Fx
    using System;
    using System.Xml.Serialization;

    /// <summary>Embeds a physics model inside another physics model or instantiates a physics model
    /// within a physics scene.</summary>
    /// <remarks>
    ///     <para>This element is used for two purposes: to hierarchically embed a physics model inside
    ///     another physics model during its definition, and to instantiate a physics model within a
    ///     physics scene. It is possible to override parameters of the contained rigid bodies and
    ///     constraints in both usages.</para>
    ///     <para>When instantiating a physics model inside a physics scene, at a minimum, the rigid
    ///     bodies that are included in the physics model can be linked with transform nodes in the
    ///     visual scene to let the physics animate meshes that are being displayed. Similarly, if a
    ///     rigid_body is kinematic instead of dynamic, an application could take transform information
    ///     from a targeted node that is being influenced by animation so that the rigid_body is moved
    ///     within its physics environment.</para>
    ///     <para>Additionally, it is possible to specify a parent attribute for the instantiated
    ///     physics model. This parent will dictate the initial position and orientation of the physics
    ///     models (and correspondingly, of its rigid bodies). The parent (or grandparent, etc.) can
    ///     also be targeted by some animation controller, to combine key-frame kinematics of nondynamic
    ///     rigid bodies with physical simulation.</para>
    ///     <para>For details about instance elements in COLLADA, see "Instantiation and External
    ///     Referencing" in Chapter 3: Schema Concepts.</para>
    /// </remarks>
    public sealed class InstancePhysicsModelTag
    {
        /// <summary>A text string value containing the scoped identifier of this element. This value must
        /// be unique within the scope of the parent element.</summary>
        /// <remarks>Optional. For details, see "Address Syntax" in Chapter 3: Schema Concepts.</remarks>
        [XmlAttribute(AttributeName = "sid")]
        public string ScopedId { get; set; }

        /// <summary>The text string name of this element.</summary>
        /// <remarks>Optional.</remarks>
        [XmlAttribute(AttributeName = "name", DataType = "token")]
        public string Name { get; set; }

        /// <summary>The URL of the location of the &lt;physics_model&gt; element to instantiate.</summary>
        /// <remarks>Required. Can refer to a local instance or external reference.</remarks>
        [XmlAttribute(AttributeName = "url", DataType = "anyURI")]
        public string Url { get; set; }

        /// <summary>Points to the id of a node in the visual scene. This allows a physics model to be
        /// instantiated under a specific transform node, which will dictate the initial position and
        /// orientation, and could be animated to influence kinematic rigid bodies.</summary>
        /// <remarks>Optional. By default, the physics model is instantiated under the world, rather
        /// than a specific transform node. This parameter is only meaningful when the parent element
        /// of the current &lt;physics_model&gt; is a &lt;physics_scene&gt;.</remarks>
        [XmlAttribute(AttributeName = "parent", DataType = "anyURI")]
        public string Parent { get; set; }


        
    }
}
