﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.ColladaDom.Physics
{
    // .NET Fx
    using System;
    using System.Xml.Serialization;

    /// <summary>Provides a means to interface with a particular &lt;rigid_body&gt; of a a
    /// &lt;physics_model&gt; that has been instantiated with &lt;instance_physics_model&gt;.</summary>
    /// <remarks>
    ///     <para>In an application that uses both physics and rich graphics, rigid bodies ultimately
    ///     set the transforms of a &lt;node&gt; in the &lt;scene&gt;. If there isn’t already a
    ///     skeleton for an instantiated physics model, the &lt;instance_rigid_body&gt; element
    ///     can be useful to connect a specific rigid body instance with a node in the visual scene.</para>
    ///     <para>
    ///         The <instance_rigid_body> element is used for three purposes:
    ///         <list>
    ///             <item>To specify the linkage to a &lt;node&gt; element</item>
    ///             <item>To optionally override parameters of a &lt;rigid_body&gt; in a specific
    ///             instance</item>
    ///             <item>To specify the initial state (linear and angular velocity) of a
    ///             &lt;rigid_body&gt; instance</item>
    ///         </list></para>
    ///     <para>For details about instance elements in COLLADA, see "Instantiation and External
    ///     Referencing" in Chapter 3: Schema Concepts.</para>
    /// </remarks>
    public sealed class InstanceRigidBodyTag
    {
        /// <summary>A text string value containing the scoped identifier of this element. This value must
        /// be unique within the scope of the parent element.</summary>
        /// <remarks>Optional. For details, see "Address Syntax" in Chapter 3: Schema Concepts.</remarks>
        [XmlAttribute(AttributeName = "sid", DataType = "sid_type")]
        public string ScopedId { get; set; }

        /// <summary>AThe text string name of this element.</summary>
        /// <remarks>Optional.</remarks>
        [XmlAttribute(AttributeName = "name", DataType = "token")]
        public string Name { get; set; }

        /// <summary>A reference to the SID of the &lt;rigid_body&gt; to instantiate.</summary>
        /// <remarks>Required.</remarks>
        [XmlAttribute(AttributeName = "body", DataType = "sidref_type")]
        public string Body { get; set; }

        /// <summary>Which &lt;node&gt; is influenced by this &lt;rigid_body&gt; instance.</summary>
        /// <remarks>Required.</remarks>
        [XmlAttribute(AttributeName = "target", DataType = "anyURI")]
        public string Target { get; set; }

        // TODO: COMMON TECHNIQUE

        /// <summary>Declares the information used to process some portion of the content.</summary>
        [XmlElement(ElementName = "technique", IsNullable = false)]
        public TechniqueTag[] Techniques { get; set; }

        /// <summary>Provides arbitrary additional information about or related to its parent element.</summary>
        [XmlElement(ElementName = "extra")]
        public ExtraTag[] Extras { get; set; }
    }
}
