﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.ColladaDom.Physics
{
    // .NET Fx
    using System;
    using System.Xml.Serialization;

    /// <summary>Specifies an environment in which physical objects are instantiated and simulated.</summary>
    /// <remarks>
    ///     <para>COLLADA allows for multiple simulations to run independently for the following main
    ///     reasons:
    ///         <list>
    ///             <item>Multiple simulations may need different global settings and they might even
    ///             run on different physics engines or on different hardware.</item>
    ///             <item>By providing such a high-level grouping mechanism, we can minimize
    ///             interactions to improve performance. For example, rigid bodies in one physics scene
    ///             are known not collide with rigid bodies of other physics scenes, so no collision
    ///             tests need to be done between them.</item>
    ///             <item>It allows for supporting multiple levels of detail (LOD)</item>
    ///         </list></para>
    ///     <para>The &lt;physics_scene&gt; element may contain techniques, extra elements, and a list
    ///     of &lt;instance_physics_model&gt; elements.</para>
    ///     <para>The "active" &lt;physics_scene&gt;s (ones that are simulated) are indicated by
    ///     instantiating them under the main &lt;scene&gt;.</para>
    /// </remarks>
    public sealed class PhysicsSceneTag
    {
        /// <summary>A text string containing the unique identifier of the element. This value must
        /// be unique within the instance document.</summary>
        /// <remarks>Optional.</remarks>
        [XmlAttribute(AttributeName = "id")]
        public string Id { get; set; }

        /// <summary>The text string name of this element.</summary>
        /// <remarks>Optional.</remarks>
        [XmlAttribute(AttributeName = "name")]
        public string Name { get; set; }

        /// <summary>Defines asset-management information regarding its parent element.</summary>
        [XmlElement(ElementName = "asset")]
        public AssetTag Asset { get; set; }

        /// <summary>Instantiates a &lt;force_field&gt; element to influence this physics scene.</summary>
        [XmlElement(ElementName = "instance_force_field")]
        public InstanceForceFieldTag[] InstanceForceFields { get; set; }

        /// <summary>Instantiates a &lt;physics_model&gt; element and allows for overriding some
        /// or all of its children.</summary>
        [XmlElement(ElementName = "instance_physics_model")]
        public InstancePhysicsModelTag[] InstancePhysicsModels { get; set; }

        /// <summary>Specifies physics-scene information for the common profile that all COLLADA
        /// implementations must support.</summary>
        [XmlElement(ElementName = "technique_common", IsNullable = false)]
        public PhysicsSceneCommonTag CommonTechnique { get; set; }

        /// <summary>Each &lt;technique&gt; specifies physics-scene information for a specific profile
        /// as designated by the &lt;technique&gt;’s profile attribute.</summary>
        [XmlElement(ElementName = "technique")]
        public TechniqueTag[] Techniques { get; set; }

        /// <summary>Provides arbitrary additional information about or related to its parent element.</summary>
        [XmlElement(ElementName = "extra")]
        public ExtraTag[] Extras { get; set; }
    }
}