﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.ColladaDom.Physics
{
    // .NET Fx
    using System;
    using System.Xml.Serialization;

    /// <summary>Constrains rigid bodies to each other or to the world.</summary>
    /// <remarks>
    ///     <para>Constraints are configurable in how they limit freedom of linear and angular
    ///     relative movement. A collection of rigid bodies and constraints can compose complex
    ///     physics models with moveable parts.</para>
    ///     <para>Building interesting physical models generally means attaching some of the rigid
    ///     bodies together, using springs, ball joints, or other types of rigid constraints.</para>
    ///     <para>COLLADA supports constraints that link two rigid bodies or a rigid body and a
    ///     coordinate frame in the scene hierarchy (for example, world space). Instead of defining
    ///     a large combination of constraint primitive elements, COLLADA offers one very flexible
    ///     element, the general six-degrees-of-freedom (DOF) constraint. Simpler constraints (for
    ///     example, linear or angular spring, ball joint, hinge) may be expressed in terms of this
    ///     general constraint.</para>
    ///     <para>A constraint is specified by:
    ///         <list>
    ///             <item>Two attachment frames, defined using a translation and orientation relative
    ///             to a rigid body’s local space or to a coordinate frame in the scene hierarchy. To
    ///             remain consistent with the rest of COLLADA, this is expressed using standard
    ///             &lt;translate&gt; and &lt;rotate&gt; elements.</item>
    ///             <item>Its degrees-of-freedom (DOF). A DOF specifies the variability along a given
    ///             axis of translation or axis of rotation, expressed in the space of the attachment
    ///             frame. For example, a door hinge typically has one degree of freedom, along a given
    ///             axis of rotation. In contrast, a slider joint has one degree of freedom along a
    ///             single axis of translation. Degrees-of-freedom and limits are specified by the
    ///             very flexible &lt;limits&gt; element.</item>
    ///         </list></para>
    /// </remarks>
    public sealed class RigidConstraintTag
    {
        /// <summary>A text string containing the scoped identifier of the &lt;rigid_constraint&gt; element. This
        /// value must be unique among its sibling elements. Associates each rigid body with a
        /// visual &lt;node&gt; when a &lt;physics_model&gt; is instantiated.</summary>
        /// <remarks>Required. For details, see "Address Syntax" in Chapter 3: Schema Concepts.</remarks>
        [XmlAttribute(AttributeName = "sid")]
        public string ScopedId { get; set; }

        /// <summary>The text string name of this element.</summary>
        /// <remarks>Optional.</remarks>
        [XmlAttribute(AttributeName = "name", DataType = "token")]
        public string Name { get; set; }

        /// <summary>Defines the attachment frame of reference (to a rigid_body or a
        /// node) within a rigid constraint.</summary>
        [XmlElement(ElementName = "ref_attachment", IsNullable = false)]
        public RefAttachmentTag RefAttachment { get; set; }

        /// <summary>Defines an attachment frame (to a rigid body or a node) within a
        /// rigid constraint.</summary>
        [XmlElement(ElementName = "attachment", IsNullable = false)]
        public AttachmentTag Attachment { get; set; }

        /// <summary>Specifies rigid-constraint information for the common profile that
        /// all COLLADA implementations must support.</summary>
        [XmlElement(ElementName = "technique_common", IsNullable = false)]
        public RigidConstraintCommonTag Common { get; set; }

        /// <summary>Each &lt;technique&gt; specifies rigid-constraint information for a
        /// specific profile as designated by the &lt;technique&gt;'s profile attribute.</summary>
        [XmlElement(ElementName = "technique")]
        public TechniqueTag[] Techniques { get; set; }

        /// <summary>Provides arbitrary additional information about or related to its parent element.</summary>
        [XmlElement(ElementName = "extra")]
        public ExtraTag[] Extras { get; set; }
    }
}
