﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.ColladaDom.Physics
{
    // .NET Fx
    using System;
    using System.Xml.Serialization;

    /// <summary>Describes components of a &lt;rigid_body&gt;.</summary>
    /// <remarks>
    ///     <para>Rigid-bodies may contain a single shape or a collection of shapes for collision
    ///     detection. Each shape may be rotated and/or translated to allow for building complex
    ///     collision shapes (a bounding shape).</para>
    ///     <para>These shapes are described by <shape> elements, each of which may contain:
    ///         <list>
    ///             <item>A &lt;physics_material&gt; definition or instance that describes the
    ///             surface properties for restitution and friction</item>
    ///             <item>Physical properties (mass or density)</item>
    ///             <item>Transforms (&lt;rotate&gt;, &lt;translate&gt;)</item>
    ///             <item>An instance or an inlined definition of a &lt;geometry&gt;</item>
    ///         </list></para>
    ///     <para>The mass properties at the shape level are irrelevant in physics simulation. It is
    ///     only the mass properties at the rigid-body level that matter. The shape element can specify
    ///     mass or density so that the parent rigid body can derive, if necessary, the body's mass
    ///     properties by integrating over its child shapes. COLLADA is designed to be an interchange
    ///     format. Typically it is easier for content creators to assign a weight to individual things
    ///     within a group than to properly assign a mass, center of mass, and moment of inertia to
    ///     group of geometries. The calculation of the mass properties is usually done during the content
    ///     export pipeline after art creation and before instantiating within the physics engine.</para>
    ///     <para>Shapes may be hollow, meaning that the mass is not distributed through the whole volume,
    ///     and is instead at the surface. In this case, density, if specified, indicates mass per
    ///     unit-of-length square. Hollow versus non-hollow (solid) affects only the calculation of
    ///     mass, inertia, and center of mass.</para>
    /// </remarks>
    public sealed class ShapeTag : MassFrameTag
    {
        /// <summary>Contains a Boolean. If true, the mass is distributed along the surface of the
        /// shape.</summary>
        [XmlElement(ElementName = "hollow")]
        public ScopedIdTag<bool> Hollow { get; set; }

        /// <summary>Contains a floating-point number specifying the mass of the shape.</summary>
        [XmlElement(ElementName = "mass")]
        public ScopedIdTag<double> Mass { get; set; }

        /// <summary>Contains a floating-point number specifying the density of the shape.</summary>
        [XmlElement(ElementName = "density")]
        public ScopedIdTag<double> Density { get; set; }

        /// <summary>Defines or references a physics_material for this shape.</summary>
        [XmlChoiceIdentifier(MemberName = "MaterialType")]
        [XmlElement(ElementName = "physics_material", Type = typeof(PhysicsMaterialTag))]
        [XmlElement(ElementName = "instance_physics_material", Type = typeof(InstancePhysicsMaterialTag))]
        public object Material { get; set; }

        /// <summary>Describes what choice of material is specified</summary>
        [XmlIgnore]
        public MaterialType MaterialType { get; set; }

        /// <summary>Defines or references a geometry for this shape.</summary>
        [XmlChoiceIdentifier(MemberName = "GeometryType")]
        [XmlElement(ElementName = "plane", Type = typeof(PlaneTag))]
        [XmlElement(ElementName = "box", Type = typeof(BoxTag))]
        [XmlElement(ElementName = "sphere", Type = typeof(SphereTag))]
        [XmlElement(ElementName = "cylinder", Type = typeof(CylinderTag))]
        [XmlElement(ElementName = "capsule", Type = typeof(CapsuleTag))]
        [XmlElement(ElementName = "instance_geometry", Type = typeof(InstanceGeometryTag))]
        public object Geometry { get; set; }

        /// <summary>Describes what choice of geometry is specified</summary>
        [XmlIgnore]
        public GeometryType GeometryType { get; set; }

        /// <summary>Provides arbitrary additional information about or related to its parent element.</summary>
        [XmlElement(ElementName = "extra")]
        public ExtraTag[] Extras { get; set; }
    }
}
