﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.ColladaDom
{
    // .NET Fx
    using System;
    using System.Xml.Serialization;

    /// <summary>Declares the input semantics of a data source and connects a consumer to that
    /// source.</summary>
    /// <remarks>
    ///     <para>The &lt;input&gt; element declares the input connections to a data source that a
    ///     consumer requires. A data source is a container of raw data that lacks semantic meaning
    ///     so that the data can be reused within the document. To use the data, a consumer declares
    ///     a connection to it with the desired semantic information.</para>
    ///     <para>The &lt;source&gt; and &lt;input&gt; elements are part of the COLLADA dataflow model.
    ///     This model is also known as stream processing, pipe, or producer-consumer. An input connection
    ///     is the dataflow path from a &lt;source&gt; to a sink (the dataflow consumers, which are
    ///     &lt;input&gt;'s parents, such as &lt;polylist&gt;).</para>
    ///     <para>In COLLADA, all inputs are driven by index values. A consumer samples an input by
    ///     supplying an index value to an input. Some consumers have multiple inputs that can share
    ///     the same index values. Inputs that have the same offset attribute value are driven by the
    ///     same index value from the consumer. This is an optimization that reduces the total number
    ///     of indexes that the consumer must store. These inputs are described in this section as
    ///     shared inputs but otherwise operate in the same manner as unshared inputs.</para>
    /// </remarks>
    public sealed class UnsharedInputTag
    {
        /// <summary>Geometric binormal (bitangent) vector</summary>
        public const string SemanticBinormal = "BINORMAL";

        /// <summary>Color coordinate vector. Color inputs are RGB (float3_type)</summary>
        public const string SemanticColor = "COLOR";

        /// <summary>Continuity constraint at the control vertex (CV). See also "Curve Interpolation" in
        /// Chapter 4: Programming Guide.</summary>
        public const string SemanticContinuity = "CONTINUITY";

        /// <summary>Raster or MIP-level input.</summary>
        public const string SemanticImage = "IMAGE";

        /// <summary>Sampler input. See also "Curve Interpolation" in Chapter 4: Programming Guide</summary>
        public const string SemanticInput = "INPUT";

        /// <summary>Tangent vector for preceding control point. See also "Curve Interpolation" in Chapter
        /// 4: Programming Guide.</summary>
        public const string SemanticInTangent = "IN_TANGENT";

        /// <summary>Sampler interpolation type. See also "Curve Interpolation" in Chapter 4: Programming
        /// Guide.</summary>
        public const string SemanticInterpolation = "INTERPOLATION";

        /// <summary>Inverse of local-to-world matrix.</summary>
        public const string SemanticInverseBindMatrix = "INV_BIND_MATRIX";

        /// <summary>Skin influence identifier</summary>
        public const string SemanticJoint = "JOINT";

        /// <summary>Number of piece-wise linear approximation steps to use for the spline segment that
        /// follows this CV. See also "Curve Interpolation" in Chapter 4: Programming Guide.</summary>
        public const string SemanticLinearSteps = "LINEAR_STEPS";

        /// <summary>Morph targets for mesh morphing</summary>
        public const string SemanticMorphTarget = "MORPH_TARGET";

        /// <summary>Weights for mesh morphing</summary>
        public const string SemanticMorphWeight = "MORPH_WEIGHT";

        /// <summary>Normal vector</summary>
        public const string SemanticNormal = "NORMAL";

        /// <summary>Sampler output. See also "Curve Interpolation" in Chapter 4: Programming Guide.</summary>
        public const string SemanticOutput = "OUTPUT";

        /// <summary>Tangent vector for succeeding control point. See also "Curve Interpolation" in
        /// Chapter 4: Programming Guide.</summary>
        public const string SemanticOutTangent = "OUT_TANGENT";

        /// <summary>Geometric coordinate vector. See also "Curve Interpolation" in Chapter 4: Programming
        /// Guide.</summary>
        public const string SemanticPosition = "POSITION";

        /// <summary>Geometric tangent vector</summary>
        public const string SemanticTangent = "TANGENT";

        /// <summary>Texture binormal (bitangent) vector</summary>
        public const string SemanticTextureBinormal = "TEXBINORMAL";

        /// <summary>Texture coordinate vector</summary>
        public const string SemanticTextureCoordinate = "TEXCOORD";

        /// <summary>Texture tangent vector</summary>
        public const string SemanticTextureTangent = "TEXTANGENT";

        /// <summary>Generic parameter vector</summary>
        public const string SemanticUV = "UV";

        /// <summary>Mesh vertex</summary>
        public const string SemanticVertex = "VERTEX";

        /// <summary>Skin influence weighting value</summary>
        public const string SemanticWeight = "WEIGHT";

        /// <summary>The offset into the list of indices defined by the parent element's &lt;p&gt; or
        /// &lt;v&gt; subelement. If two &lt;input&gt; elements share the same offset, they are indexed
        /// the same. This is a simple form of compression for the list of indices and also defines the
        /// order in which the inputs are used.</summary>
        /// <remarks>Required.</remarks>
        [XmlAttribute(AttributeName = "offset")]
        public uint Offset { get; set; }

        /// <summary>The user-defined meaning of the input connection. Required. See "Details" for the
        /// list of common &lt;input&gt; semantic attribute values enumerated in the COLLADA schema</summary>
        [XmlAttribute(AttributeName = "semantic")]
        public string Semantic { get; set; }

        /// <summary>The location of the data source.</summary>
        /// <remarks>Required.</remarks>
        [XmlAttribute(AttributeName = "source")]
        public string Source { get; set; }

        /// <summary>Which inputs to group as a single set. This is helpful when multiple inputs share
        /// the same semantics.</summary>
        /// <remarks>Optional.</remarks>
        [XmlAttribute(AttributeName = "set")]
        public uint Set { get; set; }
    }
}
