﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics.TestGame.Screens
{
    // .NET Fx
    using System;
    using System.Collections.Generic;

    // XNA
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Graphics;

    // Local
    using JohnWells.GameDynamics.Geometries;
    using JohnWells.GameDynamics.Constraints;

    internal sealed class BeachBallPileScreen : TestScreen
    {
        #region Fields
        // Private variables
        private Simulation simulation;

        // Content
        private ModelMesh ball; 
        #endregion

        public BeachBallPileScreen(MyGame game)
            : base(game, "Content")
        {
        }

        #region Properties
        public override string Description
        {
            get { return "Beach Ball Pile - Falling balls that are constrained to the space above a plane"; }
        }
        #endregion

        #region Methods
        public override void Initialize()
        {
            base.Initialize();
            this.simulation = new Simulation();
            var r = new Random();

            // Create a bunch of spheres
            for (var index = 0; index < 80; index++)
            {
                var yaw = (float)r.NextDouble() * MathHelper.TwoPi;
                var pitch = (float)r.NextDouble() * MathHelper.TwoPi;
                var roll = (float)r.NextDouble() * MathHelper.TwoPi;
                var orientation = Quaternion.CreateFromYawPitchRoll(yaw, pitch, roll);
                orientation.Normalize();

                var angle = (float)r.NextDouble() * MathHelper.TwoPi;
                var distance = (float)r.NextDouble() * 4f;
                var x = (float)Math.Sin(angle) * distance;
                var y = (float)r.NextDouble() * 32f + 4f;
                var z = (float)Math.Cos(angle) * distance;
                var state = new Transform(new Vector3(x, y, z), orientation);
                var body = new Sphere(state, Material.Rubber, 1f);
                this.simulation.Add(body);
                this.simulation.Add(new PlaneConstraint(body, new Plane(Vector3.Up, 0f), Material.Rubber));
            }

            // Create a three-layer wall of balls to contain the falling ones
            for (var y = 1f; y < 6f; y += 2f)
                for (var angle = 0f + (y / 5f); angle < MathHelper.TwoPi + (y / 5f); angle += MathHelper.PiOver4 / 5f)
                {
                    var x = (float)Math.Sin(angle) * 16f;
                    var z = (float)Math.Cos(angle) * 16f;
                    var state = new Transform(new Vector3(x, y, z), Quaternion.Identity);
                    var body = new Sphere(state, Material.Rubber, 1f);
                    body.IsStatic = true;
                    this.simulation.Add(body);
                }

            // To get bodies to a non-intersecting point
            //this.Simulation.Update(1f / 16f);
        }

        protected override void LoadContent()
        {
            base.LoadContent();

            this.ball = this.Content.Load<Model>("BeachBall").Meshes["Ball"];
            foreach (BasicEffect item in this.ball.Effects)
            {
                item.View = Matrix.CreateLookAt(
                    new Vector3(45f, 36f, 5f),
                    new Vector3(0f, 0f, 0f),
                    Vector3.Up);

                item.Projection = Matrix.CreatePerspectiveFieldOfView(
                    MathHelper.ToRadians(45f),
                    this.GraphicsDevice.Viewport.AspectRatio,
                    1f,
                    10000f);

                item.EnableDefaultLighting();
            }

            GC.Collect();
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);
            this.simulation.Update(gameTime.ElapsedGameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            this.GraphicsDevice.RenderState.DepthBufferEnable = true;
            foreach (var body in this.simulation.GetEnumerator())
            {
                foreach (BasicEffect effect in this.ball.Effects)
                    effect.World = body.State;
                
                this.ball.Draw();
            }
        }
        #endregion
    }
}