/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics.TestGame.Screens
{
    // .NET Fx
	using System;
    using System.Collections.Generic;
    
    // XNA
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Graphics;
    using Microsoft.Xna.Framework.Input;

    // Local
    using JohnWells.GameFx;

    internal sealed class SelectionScreen : GameScreen
    {
        #region Fields
        // Constants
		private const string Banner = "JohnWells.GameDynamics Tests\r\n\t[T] Select a test\r\n\t[Space] Reset current test\r\n\t[S] Display stats in console\r\n\t[Esc/Back] Exit";
        private const string Warning = "< 60 frames/sec";

        // Input
        private KeyboardState previousKeys;
        private GamePadState previousPad;

        // Child screens
        private readonly TestScreen[] tests;
        private int index;

        // Content
        private SpriteFont font;
        #endregion

        public SelectionScreen(MyGame game)
            : base(game, "Content")
        {
            this.DrawOrder = 20;
            this.UpdateOrder = 10;

            // Initialize the keyboard and gamepad state
            this.previousKeys = Keyboard.GetState();
            this.previousPad = GamePad.GetState(PlayerIndex.One);

            // Create the screen instances (now instead of later..)
            this.tests = new TestScreen[]
                {
                    // These screens are boring and only useful for testing specific things:
                    // (See notes in each screen's code on how to interpret the results)
                    // new CDTestScreen(game),
                    // new GravityTestScreen(game),
                    // new PlaneConstraintScreen(game),
                    //new ForceScreen(game),

                    // Fun screens below
                    new NewtonsCradleScreen(game),
                    new OpposingBallsScreen(game),
                    new OpposingBoxesScreen(game),
                    new BreakingWallScreen(game),
                    new FallingBlockScreen(game),
                    new BeachBallPileScreen(game),
                    new DroppingBlocksScreen(game)
                };

            // Pick the first screen
            this.index = 0;
            this.Show(this.tests[this.index]);
        }

        #region Methods
        protected override void LoadContent()
        {
            base.LoadContent();

            // Load the font we'll use to draw information the text
            this.font = this.Content.Load<SpriteFont>("Tahoma14");
        }

        public override void Update(GameTime gameTime)
        {
            var keys = Keyboard.GetState();
            var pad = GamePad.GetState(PlayerIndex.One);

            // Cycle screens based on keyboard input (Page Up/Page Down)
            if (this.previousKeys.IsKeyUp(Keys.T) && keys.IsKeyDown(Keys.T))
            {
                // Cycle up in the list
                this.index += 1;
                this.index %= this.tests.Length;

                // Show the new screen
                this.ChildScreen.Close();
                this.Show(this.tests[index]);
            }

            // Reset the current test
            if (this.previousKeys.IsKeyUp(Keys.Space) && keys.IsKeyDown(Keys.Space))
            {
                // Reset if we have a child screen (we always will...)
                var child = this.ChildScreen;
                if (child != null)
                    child.Initialize();
            }

			// Display statistics
			if (this.previousKeys.IsKeyUp(Keys.S) && keys.IsKeyDown(Keys.S))
			{
				// Create a little garbage in the name of printing out statistics - only works in debug (TRACE) build
				Console.WriteLine(string.Format("Stats for {0}\r\nUpdates: {1}\r\n(Counter Current/Last/Total)\r\nPenetrations: {2}/{3}/{4}\r\nForces: {5}/{6}/{7}",
					DateTime.Now.ToLongTimeString(),
					Instrumentation.Total.Updates,
					Instrumentation.Current.PenetrationsDetected, Instrumentation.Last.PenetrationsDetected, Instrumentation.Total.PenetrationsDetected,
					Instrumentation.Current.ForcesApplied, Instrumentation.Last.ForcesApplied, Instrumentation.Total.ForcesApplied));
			}

            // Exit
            if ((this.previousKeys.IsKeyUp(Keys.Escape) && keys.IsKeyDown(Keys.Escape)) ||
                (this.previousPad.IsButtonUp(Buttons.Back) && pad.IsButtonDown(Buttons.Back)))
            {
                // Bammo! Everything should clean up on it's own...
                this.Game.Exit();
            }

            this.previousKeys = keys;
            this.previousPad = pad;
        }

        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            this.SpriteBatch.Begin();

            var currentTest = this.ChildScreen as TestScreen;

            // Draw a title and the current test
            this.SpriteBatch.DrawString(this.font, SelectionScreen.Banner, new Vector2(10f, 10f), Color.AliceBlue);
            this.SpriteBatch.DrawString(this.font, (currentTest == null) ? string.Empty : currentTest.Description, new Vector2(340f, 10f), Color.AntiqueWhite);

            // Draw the version number of this test app - It's hardcoded but that's okay with me...
            Viewport viewport = this.GraphicsDevice.Viewport;
            if (gameTime.IsRunningSlowly)
            {
                Vector2 size = this.font.MeasureString(SelectionScreen.Warning);
                this.SpriteBatch.DrawString(
                    this.font,
                    SelectionScreen.Warning,
                    new Vector2(viewport.Width - size.X - 10f, 10f),
                    Color.Tomato);
            }

            this.SpriteBatch.End();
        } 
        #endregion
    }
}
