﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics.CD
{
    // .NET Fx
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;

    // Local
    using JohnWells.GameDynamics.Geometries;

    /// <summary>Manages an array of narrow phase collision detection algorithms</summary>
    /// <remarks>Falls back to MPR, or the configured default, for unassociated geometry</remarks>
    public sealed partial class Arbiter : ICollider
    {
        #region Fields
        // Private variables
        private readonly ICollider fallBack;
        private readonly List<Entry> cache;
        #endregion

        public Arbiter()
            : this(null)
        {
        }

        public Arbiter(ICollider fallBack)
        {
            this.fallBack = fallBack ?? new Mpr();
            this.cache = new List<Entry>();
        }

        #region Methods
        /// <summary>Associates type a (which must be a subclass of JohnWells.GameDynamics.Geometries.Convex) with the given
        /// collision detection algorithm</summary>
        public void Associate(ICollider cd, Type collider)
        {
            this.Associate(cd, collider, collider);
        }

        /// <summary>Associates types a and b (which must be subclasses of JohnWells.GameDynamics.Body) with the given
        /// collision detection arbiter</summary>
        /// <remarks>Optimize this arbiter by associating geometry types in most-used order</remarks>
        public void Associate(ICollider cd, Type a, Type b)
        {
            // Add an entry forwards
            var ab = new Entry(cd, a, b);
            this.cache.Add(ab);

            // If needed, add an entry backwards
            if (!object.ReferenceEquals(a, b))
            {
                var ba = new Entry(cd, b, a);
                this.cache.Add(ba);
            }
        }

        /// <summary>Generates a new instance of the general arbiter which is preconfigured to
        /// handle all built-in geometries using generic and specialized algorithms</summary>
        public static ICollider CreateDefault()
        {
            var result = new Arbiter();

            // Shortcuts for readability
            var box = typeof(Box);
            var sphere = typeof(Sphere);
            
            // Make the default associations
            result.Associate(new SphereCollider(), sphere);
            result.Associate(new Mpr(), box);

            return result;
        }
        #endregion

        #region ICollider Members
        /// <summary>Returns the point of deepest contact if the two bodies provided are in
        /// contact in their current state</summary>
        /// <remarks>Different implementations may provide contact data that is not
        /// completely accurate, but the data should at the very least be repeatable</remarks>
        public Contact? Detect(Body a, Body b)
        {
            var aType = a.GetType();
            var bType = b.GetType();

            // In production code you should have an ICollider instance associated with all of your geometry.
            // If you don't want two geometries to have collision detection, associate them with
            // the no-op ICollider or use the no-op ICollider as the default when creating an instance
            // of this class....

            // Find the configured ICollider for these two bodies (if possible)
            foreach (var entry in this.cache)
            {
                if (object.ReferenceEquals(entry.A, aType) && object.ReferenceEquals(entry.B, bType))
                    return entry.CD.Detect(a, b);
            }
            
            // Use the default CD for unassociated types
            return this.fallBack.Detect(a, b);
        }
        #endregion
    }
}
