/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

#define INLINE_MATH

namespace JohnWells.GameDynamics.CD
{
    // .NET Fx
    using System;
    using System.Diagnostics;
    using System.Runtime.InteropServices;

    // XNA
    using Microsoft.Xna.Framework;

    /// <summary>Describes an immutable generic contact manifold</summary>
    [StructLayout(LayoutKind.Auto)]
    public struct Contact
    {
        #region Fields
        /// <summary>Gets the position of the mean point of this contact</summary>
        /// <remarks>Value is in the world frame</remarks>
        public readonly Vector3 Position;

        /// <summary>Gets the surface normal at the position of contact</summary>
        /// <remarks>Points in the direction of the minimum translational distance, value is
        /// in the world frame</remarks>
        public readonly Vector3 Normal;

        /// <summary>Gets the minimum translational distance that must be moved in the
        /// direction of 'Normal' at 'Position' to resolve this contact, value is in the
        /// world frame</summary>
        public readonly float Depth;
        #endregion

        public Contact(Vector3 position, Vector3 normal, float depth)
            : this(ref position, ref normal, depth)
        {
        }

        internal Contact(ref Vector3 position, ref Vector3 normal, float depth)
        {
            #region Assertions
            Debug.Assert(!float.IsNaN(position.X) && !float.IsInfinity(position.X));
            Debug.Assert(!float.IsNaN(position.Y) && !float.IsInfinity(position.Y));
            Debug.Assert(!float.IsNaN(position.Z) && !float.IsInfinity(position.Z));
            Debug.Assert(!float.IsNaN(normal.X) && !float.IsInfinity(normal.X));
            Debug.Assert(!float.IsNaN(normal.Y) && !float.IsInfinity(normal.Y));
            Debug.Assert(!float.IsNaN(normal.Z) && !float.IsInfinity(normal.Z));
            Debug.Assert(normal.Length() > 0.99f);
            Debug.Assert(normal.Length() < 1.01f);
            Debug.Assert(!float.IsNaN(depth) && !float.IsInfinity(depth));
            Debug.Assert(depth >= 0f); 
            #endregion

            this.Position = position;
            this.Depth = depth;

#if !INLINE_MATH
            // TODO: I'd rather remove this -> Someone passed us a normal, it's their job to make sure it's good...
            Vector3.Normalize(ref normal, out this.Normal);
#else
            // TODO: I'd rather remove this -> Someone passed us a normal, it's their job to make sure it's good...
            float scale = 1f / (float)Math.Sqrt(normal.X * normal.X + normal.Y * normal.Y + normal.Z * normal.Z);
            this.Normal.X = normal.X * scale;
            this.Normal.Y = normal.Y * scale;
            this.Normal.Z = normal.Z * scale;
#endif

            #region Assertions
            Debug.Assert(!float.IsNaN(this.Normal.X) && !float.IsInfinity(normal.X));
            Debug.Assert(!float.IsNaN(this.Normal.Y) && !float.IsInfinity(normal.Y));
            Debug.Assert(!float.IsNaN(this.Normal.Z) && !float.IsInfinity(normal.Z));
            Debug.Assert(normal.Length() > 0f);
            #endregion
        }
    }
}
