/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics.CD
{
    // XNA
    using Microsoft.Xna.Framework;

    /// <summary>A Minkowski Portal Refinement based contact algorithm</summary>
    /// <remarks>Based entirely upon the XenoCollide algorithm from Game Programming Gems 7</remarks>
    public sealed class Mpr : ICollider
    {
        #region Fields
        // Constants
        private const int MaxIterations = 32;
        private const float Epsilon = 1e-3f;
        #endregion

        #region Methods
        /// <summary>Swaps two value types</summary>
        private static void Swap<T>(ref T a, ref T b)
            where T : struct
        {
            var temp = a;
            a = b;
            b = temp;
        }
        #endregion

        #region ICollider Members
        public Contact? Detect(Body bodyA, Body bodyB)
        {
            /* For more information see:
             * Snethen, Gary/ "2.5 XenoCollide: Complex Collision Made Simple" Game Programming
             * Gems 7 - Charles River Media, 2008. 165-178 */

            var a = new SupportMap(bodyA);
            var b = new SupportMap(bodyB);

            // v0 = Any arbitrary point deep within the Minkowski Difference
            var v0a = bodyA.State.Translation;
            var v0b = bodyB.State.Translation;
            var v0 = v0b - v0a;

            // Edge case: 'a' and 'b' are body-aligned and overlapping
            Vector3 normal;
            if (v0 == Vector3.Zero)
                normal = Vector3.Right;
            else
                normal = -v0;

            // v1 = Support in direction of configuration space origin
            Vector3 v1a, v1b;
            var v1 = a.MinkowskiDifference(b, normal, out v1a, out v1b);
            if (Vector3.Dot(v1, normal) <= 0f)
                return null;

            // Edge case: 'a' and 'b' are edge-aligned (overlapping)
            normal = Vector3.Cross(v1, v0);
            if (normal == Vector3.Zero)
            {
                normal = v1 - v0;
                normal.Normalize();

                var plane = new Plane(normal, Vector3.Dot(-v1b, normal));
                var depth = plane.DotCoordinate(v1a) * -1f;

                return new Contact(v1a, normal, depth);
            }

            // v2 = Support perpendicular to line {v1, v0}
            var v2 = a.MinkowskiDifference(b, normal);
            if (Vector3.Dot(v2, normal) <= 0f)
                return null;

            // Determine which side of plane {v1, v0, v2} the configuration space origin is
            // (If the origin is on the '-' side of the plane, reverse the plane)
            normal = Vector3.Cross(v2 - v0, v1 - v0);
            if (Vector3.Dot(v0, normal) > 0f)
            {
                Mpr.Swap(ref v1, ref v2);
                normal = -normal;
            }

            var iteration = 1;

            // Phase One: Identify a portal (by finding v3)
            Vector3 v3;
            while (true)
            {
                // Have we passed the allowed number of iterations?
                if (iteration++ > Mpr.MaxIterations)
                    return null;

                // Obtain the support point in a direction perpendicular to the existing plane
                v3 = a.MinkowskiDifference(b, normal);
                if (Vector3.Dot(v3, normal) <= 0f)
                    return null;

                // If origin is outside {v1, v0, v3} then eliminate v2
                if (Vector3.Dot(Vector3.Cross(v1, v3), v0) < 0f)
                {
                    v2 = v3;
                    normal = Vector3.Cross(v1 - v0, v3 - v0);
                    continue;
                }

                // If origin is outside {v3, v0, v2} then eliminate v1
                if (Vector3.Dot(Vector3.Cross(v3, v2), v0) < 0f)
                {
                    v1 = v3;
                    normal = Vector3.Cross(v3 - v0, v2 - v0);
                    continue;
                }

                // Yay! We found a portal!
                break;
            }

            // Phase Two: Refine the portal (We are now inside of a wedge...)
            Vector3? position = null;
            while (true)
            {
                // Have we passed the allowed number of iterations?
                if (iteration++ > Mpr.MaxIterations)
                    return null;

                // Compute normal of the wedge face
                // If the origin is inside the wedge, we have a hit - We only need to calculate this once
                // while we search for the correct depth of this contact
                normal = Vector3.Normalize(Vector3.Cross(v2 - v1, v3 - v1));
                if (Vector3.Dot(v1, normal) > 0f)
                {
                    // Hit: Use the portal we found to compute the barycentric coordinates of the origin
                    float b0 = Vector3.Dot(Vector3.Cross(v1, v2), v3);
                    float b1 = Vector3.Dot(Vector3.Cross(v3, v2), v0);
                    float b2 = Vector3.Dot(Vector3.Cross(v0, v1), v3);
                    float b3 = Vector3.Dot(Vector3.Cross(v2, v1), v0);
                    float sum = b0 + b1 + b2 + b3;

                    if (sum <= 0f)
                    {
                        b0 = 0f;
                        b1 = Vector3.Dot(Vector3.Cross(v2, v3), normal);
                        b2 = Vector3.Dot(Vector3.Cross(v3, v1), normal);
                        b3 = Vector3.Dot(Vector3.Cross(v1, v2), normal);
                        sum = b1 + b2 + b3;
                    }

                    // Hit
                    position = (b0 * v0 + b1 * v1 + b2 * v2 + b3 * v3) / sum;
                }

                // Find the support point in the direction of the wedge face
                var v4 = a.MinkowskiDifference(b, normal);

                float delta = Vector3.Dot(v4 - v3, normal);
                float depth = Vector3.Dot(v4, normal);

                // If the boundary is thin enough or the origin is outside the support plane for the newly discovered vertex, then we can terminate
                if (delta < Mpr.Epsilon)
                {
                    // If the delta is too small and we still haven't found contact, return a miss
                    if (!position.HasValue || depth <= 0f)
                        return null;

                    // Return the fully formed contact
                    return new Contact(position.Value, normal, depth);
                }

                // The new point v4 replaces one of our existing vertices
                if (Vector3.Dot(Vector3.Cross(v4, v1), v0) < 0f)
                    if (Vector3.Dot(Vector3.Cross(v4, v2), v0) < 0f)
                    {
                        v1 = v4;
                    }
                    else
                    {
                        v3 = v4;
                    }
                else
                    if (Vector3.Dot(Vector3.Cross(v4, v3), v0) < 0f)
                    {
                        v2 = v4;
                    }
                    else
                    {
                        v1 = v4;
                    }
            }
        }
        #endregion
    }
}