/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

//#define INLINE_MATH

namespace JohnWells.GameDynamics.CD
{
    // .NET Fx
    using System;
    using System.Diagnostics;
    using System.Runtime.InteropServices;

    // XNA
    using Microsoft.Xna.Framework;

    /// <summary>An immutable structure that wraps the support point sampling of a surface and a state of that surface
    /// to provide a world coordinate support mapping function</summary>
    /// <remarks>Also provided are functions to work on the Minkowski Difference of two sampled surfaces, which is used
    /// in a number of collision detection algorithms</remarks>
    [StructLayout(LayoutKind.Auto)]
    internal struct SupportMap
    {
        #region Fields
        private readonly Body body;
        private readonly Transform state;
        #endregion

        public SupportMap(Body body)
        {
            #region Assertions
            Debug.Assert(body != null);
            #endregion

            this.body = body;
            this.state = body.State;
        }

        #region Methods
        /// <summary>Samples the convex hull of the Minkowski Difference defined by this instance and the given other instance</summary>
        /// <param name="direction">A direction vector that does not need to be normalized</param>
        /// <remarks>Description at <see cref="http://www.pfirth.co.uk/minkowski.html"/></remarks>
        public Vector3 MinkowskiDifference(SupportMap other, Vector3 direction)
        {
            Vector3 result, support, otherSupport;
            this.MinkowskiDifference(ref other, ref direction, out result, out support, out otherSupport);
            return result;
        }

        /// <summary>Samples the convex hull of the Minkowski Difference defined by this instance and the given other instance</summary>
        /// <param name="direction">A direction vector that does not need to be normalized</param>
        /// <remarks>Description at <see cref="http://www.pfirth.co.uk/minkowski.html"/></remarks>
        public Vector3 MinkowskiDifference(SupportMap other, Vector3 direction, out Vector3 support, out Vector3 otherSupport)
        {
            Vector3 result;
            this.MinkowskiDifference(ref other, ref direction, out result, out support, out otherSupport);
            return result;
        }

        /// <summary>Samples the convex hull of the Minkowski Difference defined by this instance and the given other instance</summary>
        /// <param name="direction">A direction vector that does not need to be normalized</param>
        /// <remarks>Description at <see cref="http://www.pfirth.co.uk/minkowski.html"/></remarks>
        public void MinkowskiDifference(ref SupportMap other, ref Vector3 direction, out Vector3 result)
        {
            Vector3 support, otherSupport;
            this.MinkowskiDifference(ref other, ref direction, out result, out support, out otherSupport);
        }

        /// <summary>Samples the convex hull of the Minkowski Difference defined by this instance and the given other instance</summary>
        /// <param name="direction">A direction vector that does not need to be normalized</param>
        /// <remarks>Description at <see cref="http://www.pfirth.co.uk/minkowski.html"/></remarks>
        public void MinkowskiDifference(ref SupportMap other, ref Vector3 direction, out Vector3 result, out Vector3 support, out Vector3 otherSupport)
        {
            #region Assertions
            Debug.Assert(!float.IsNaN(direction.X) && !float.IsInfinity(direction.X));
            Debug.Assert(!float.IsNaN(direction.Y) && !float.IsInfinity(direction.Y));
            Debug.Assert(!float.IsNaN(direction.Z) && !float.IsInfinity(direction.Z));
            Debug.Assert(direction.Length() > 0f);
            #endregion
            
#if !INLINE_MATH
            // Support on the hull of the Minkowski Difference = B(n) - A(-n)
            otherSupport = other.Support(direction);
            support = this.Support(-direction);
            result = otherSupport - support;
#else
            // support = <other.Support(direction)> - this.Support(-direction)
            other.Support(ref direction, out otherSupport);

            // support = other.Support(direction) - <this.Support(-direction)>
            Vector3 otherDirection = new Vector3(-direction.X, -direction.Y, -direction.Z);
            this.Support(ref otherDirection, out support);

            // support = <other.Support(direction) - this.Support(-direction)>
            result = new Vector3(otherSupport.X - support.X, otherSupport.Y - support.Y, otherSupport.Z - support.Z);
#endif

            #region Assertions
            Debug.Assert(!float.IsNaN(support.X) && !float.IsInfinity(support.X));
            Debug.Assert(!float.IsNaN(support.Y) && !float.IsInfinity(support.Y));
            Debug.Assert(!float.IsNaN(support.Z) && !float.IsInfinity(support.Z));
            #endregion
        }

        /// <summary>Finds the point (on the surface of this shape) most in the given direction</summary>
        /// <param name="direction">A direction vector that does not need to be normalized</param>
        /// <remarks>Coordinates are in the world frame</remarks>
        public Vector3 Support(Vector3 direction)
        {
            Vector3 result;
            this.Support(ref direction, out result);
            return result;
        }

        /// <summary>Finds the point (on the surface of this shape) most in the given direction</summary>
        /// <param name="direction">A direction vector that does not need to be normalized</param>
        /// <remarks>Coordinates are in the world frame</remarks>
        public void Support(ref Vector3 direction, out Vector3 support)
        {
            #region Assertions
            Debug.Assert(!float.IsNaN(direction.X) && !float.IsInfinity(direction.X));
            Debug.Assert(!float.IsNaN(direction.Y) && !float.IsInfinity(direction.Y));
            Debug.Assert(!float.IsNaN(direction.Z) && !float.IsInfinity(direction.Z));
            Debug.Assert(direction.Length() > 0f);
            #endregion

#if !INLINE_MATH
            // Rotate the input world direction to a local direction
            var conjugate = Quaternion.Conjugate(this.state.Orientation);
            var localDirection = Vector3.Transform(direction, conjugate);
            
            // Find local support using local direction
            var localSupport = this.body.Support(localDirection);

            // Transform the local support to a world support
            support = Vector3.Transform(localSupport, this.state.Orientation) + this.state.Translation;
#else
            bla!
            Vector3 localNormal, localSupport;
            Vector3.Transform(ref normal, ref this.State.Orientation
            this.State.RotateToLocal(ref normal, out localNormal);
            this.Surface.Support(ref localNormal, out localSupport);
            this.State.Transform(ref localSupport, out support);
#endif

            #region Assertions
            Debug.Assert(!float.IsNaN(support.X) && !float.IsInfinity(support.X));
            Debug.Assert(!float.IsNaN(support.Y) && !float.IsInfinity(support.Y));
            Debug.Assert(!float.IsNaN(support.Z) && !float.IsInfinity(support.Z));
            #endregion
        }
        #endregion
    }
}
