/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics.Constraints
{
    // .NET Fx
    using System.Diagnostics;

    // XNA
    using Microsoft.Xna.Framework;

    // Local
    using JohnWells.GameDynamics.CD;

    /// <summary>Constrains a body to the positive side of a given plane, with an optional material
    /// to specify how bodies react to impacts</summary>
    public sealed class PlaneConstraint : IConstrained
    {
        #region Fields
        // Private variables
        private readonly Body body;
        private readonly Plane constraintPlane;
        private readonly Vector3 supportNormal;
        private readonly Material? material;
        #endregion

        public PlaneConstraint(Body body, Plane plane)
            : this(body, plane, null)
        {
        }

        public PlaneConstraint(Body body, Plane plane, Material material)
            : this(body, plane, (Material?)material)
        {
        }

        private PlaneConstraint(Body body, Plane plane, Material? material)
        {
            #region Assertions
            Debug.Assert(body != null); 
            #endregion

            this.body = body;
            this.material = material;

            Vector3 normal = Vector3.Normalize(plane.Normal);
            this.supportNormal = Vector3.Negate(normal);
            this.constraintPlane = new Plane(normal, plane.D);
        }

        #region IConstraint Members
        void IConstrained.ApplyForces()
        {
            // Early out if we don't need to figure force/torque due to this collision
            if (!this.material.HasValue)
                return;

            // Find the deepest point of our body into the constrained plane
            var map = new SupportMap(this.body);
            var support = map.Support(this.supportNormal);

            // Is this constraint violated?
            var distance = this.constraintPlane.DotCoordinate(support);
            if (distance >= 0f)
                return;

            var center = this.body.State.Translation;

            // Is the support point we found moving towards the constraint plant?
            var supportRelative = support - center;
            var velocity = this.body.Velocity + Vector3.Cross(supportRelative, this.body.Spin);
            float relativeSpeed = Vector3.Dot(this.constraintPlane.Normal, velocity);
            if (relativeSpeed >= 0f)
                return;

            // TODO: Add bounce!
        }

        void IConstrained.Satisfy()
        {
            // Find the deepest point of our body into the constrained plane
            var map = new SupportMap(this.body);
            var support = map.Support(this.supportNormal);

            // Is this constraint violated?
            var distance = this.constraintPlane.DotCoordinate(support);
            if (distance >= 0f)
                return;

            // Correct position error (entirely linear)
            var translationError = distance * this.constraintPlane.Normal;
            this.body.State = new Transform(this.body.State.Translation - translationError, this.body.State.Orientation);

            var center = this.body.State.Translation;

            // Is the support point we found moving towards the constraint plant?
            var supportRelative = support - center;
            var velocity = this.body.Velocity + Vector3.Cross(supportRelative, this.body.Spin);
            float relativeSpeed = Vector3.Dot(this.constraintPlane.Normal, velocity);
            if (relativeSpeed >= 0f)
                return;

            // Correct the linear velocity error
            Vector3 velocityError = relativeSpeed * this.constraintPlane.Normal;
            this.body.Velocity = this.body.Velocity - velocityError;
        }
        #endregion
    }
}
