﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics.Constraints
{
    // .NET Fx
    using System.Diagnostics;

    // XNA
    using Microsoft.Xna.Framework;

	/// <summary>Connects a body to a world point using a spring</summary>
	public sealed class TetherConstraint : IConstrained
	{
		#region Fields
		// Private variables
		private readonly Body body;
		private readonly Vector3 relative, anchor;
		private readonly float restLength, springStrength, dampingStrength;
		#endregion

		public TetherConstraint(Body body, Vector3 relative, Vector3 anchor, float restLength, float strength)
		{
			#region Assertions
			Debug.Assert(body != null);
			Debug.Assert(!float.IsNaN(relative.X) && !float.IsInfinity(relative.X));
			Debug.Assert(!float.IsNaN(relative.Y) && !float.IsInfinity(relative.Y));
			Debug.Assert(!float.IsNaN(relative.Z) && !float.IsInfinity(relative.Z));
			Debug.Assert(!float.IsNaN(anchor.X) && !float.IsInfinity(anchor.X));
			Debug.Assert(!float.IsNaN(anchor.Y) && !float.IsInfinity(anchor.Y));
			Debug.Assert(!float.IsNaN(anchor.Z) && !float.IsInfinity(anchor.Z));
			Debug.Assert(!float.IsNaN(restLength) && !float.IsInfinity(restLength));
			Debug.Assert(restLength >= 0f);
			Debug.Assert(strength >= 0f);
			//Debug.Assert(strength <= 1f); 
			#endregion

			this.body = body;
			this.relative = relative;
			this.anchor = anchor;
			this.restLength = restLength;
			this.springStrength = strength;
			this.dampingStrength = strength * -0.5f;
		}

		#region IConstrained Members
		void IConstrained.ApplyForces()
		{
			var position = Vector3.Transform(this.relative, this.body.State.Orientation) + this.body.State.Translation;
			var spring = this.anchor - position;
			var length = spring.Length();

			// Spring
			var displacement = length - this.restLength;
			var normal = spring / length;
			var force = normal * (displacement * this.springStrength);
			this.body.ApplyForce(this.relative, force);

			// Damper
			var velocity = this.body.Velocity + Vector3.Cross(this.relative, this.body.Spin);
			var damper = normal * (Vector3.Dot(normal, velocity) * this.dampingStrength);
			this.body.ApplyForce(this.relative, damper);
		}

		void IConstrained.Satisfy()
		{
		}
		#endregion
	}
}
