/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics.Geometries
{
    // .NET Fx
    using System.Diagnostics;

    // XNA
    using Microsoft.Xna.Framework;

    /// <summary>A box</summary>
    /// <remarks>http://en.wikipedia.org/wiki/Hexahedron</remarks>
    public sealed class Box : Body
    {
        #region Fields
        // Private variables
        private readonly float x, y, z, radius;
        #endregion

        public Box(Transform state, Material material, Vector3 scale)
            : base(state, material)
        {
            #region Assertions
            Debug.Assert(!float.IsNaN(scale.X) && !float.IsInfinity(scale.X));
            Debug.Assert(!float.IsNaN(scale.Y) && !float.IsInfinity(scale.Y));
            Debug.Assert(!float.IsNaN(scale.Z) && !float.IsInfinity(scale.Z));
            #endregion

            // We store half-scale for ease of use
            this.x = scale.X * 0.5f;
            this.y = scale.Y * 0.5f;
            this.z = scale.Z * 0.5f;

            // Create the local bounds of this (origin centered) box
            var min = new Vector3(-this.x, -this.y, -this.z);
            var max = new Vector3(this.x, this.y, this.z);
            var box = new BoundingBox(min, max);
            var sphere = BoundingSphere.CreateFromBoundingBox(box);
			this.radius = sphere.Radius;

            // Total mass is 'h x w d x Density'
            var mass = scale.X * scale.Y * scale.Z * material.Density;

            // Millington, Ian. "A.3 Common Shapes" Game Physics Engine Development - Elsevier, 2007. 432-433
            var inertia = Vector3.Zero;
            inertia.X = (1f / 12f) * mass * 2f;
            inertia.Y = (1f / 12f) * mass * 2f;
            inertia.Z = (1f / 12f) * mass * 2f;

            this.MassInverse = 1f / mass;
            this.InertiaInverse = Vector3.One / inertia;

            #region Assertions
            Debug.Assert(!float.IsNaN(this.InertiaInverse.X) && !float.IsInfinity(this.InertiaInverse.X));
            Debug.Assert(!float.IsNaN(this.InertiaInverse.Y) && !float.IsInfinity(this.InertiaInverse.Y));
            Debug.Assert(!float.IsNaN(this.InertiaInverse.Z) && !float.IsInfinity(this.InertiaInverse.Z));
            Debug.Assert(!float.IsNaN(this.MassInverse) && !float.IsInfinity(this.MassInverse));
            #endregion
        }

        #region Properties
        /// <summary>Gets the local axial half-extents of this box</summary>
        public Vector3 HalfExtents
        {
            get { return new Vector3(this.x, this.y, this.z); }
        }

		/// <summary>Gets the radius of a bounding sphere which surrounds body</summary>
		public override float Radius
		{
			get { return this.radius; }
		}
        #endregion

        #region Methods
        /// <summary>Finds the point (on the surface of this shape) most in the given direction</summary>
        /// <param name="direction">A direction vector that does not need to be normalized</param>
        /// <remarks>Coordinates are in the local frame</remarks>
        public override void Support(ref Vector3 direction, out Vector3 support)
        {
            #region Assertions
            Debug.Assert(!float.IsNaN(direction.X) && !float.IsInfinity(direction.X));
            Debug.Assert(!float.IsNaN(direction.Y) && !float.IsInfinity(direction.Y));
            Debug.Assert(!float.IsNaN(direction.Z) && !float.IsInfinity(direction.Z));
            Debug.Assert(direction.Length() > 0f);
            #endregion

            // See http://www.win.tue.nl/~gino/solid/jgt98convex.pdf by Gino van den Bergen
            support = new Vector3(
                (direction.X > 0f) ? this.x : -this.x,
                (direction.Y > 0f) ? this.y : -this.y,
                (direction.Z > 0f) ? this.z : -this.z);
        }
        #endregion
    }
}
