/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics.Geometries
{
    // .NET Fx
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;

    // XNA
    using Microsoft.Xna.Framework;

    /// <summary>A convex collection of points that describe a hull</summary>
    /// <remarks>This class is for run-time use and does not provide a means to find the hull of your point set - Consider
    /// using the built-in pipeline or QHull to do this before passing the points to this class - Also make sure your points
    /// describe a shape whose center of mass is the local origin</remarks>
    public sealed class ConvexHull : Body
    {
        #region Fields
        // Private variables
        private readonly IEnumerable<Vector3> points;
		private readonly float radius;
        #endregion

        public ConvexHull(Transform state, Material material, IEnumerable<Vector3> convexSet)
            : base(state, material)
        {
            #region Assertions
			Debug.Assert(convexSet != null);
            #endregion

			var sphere = BoundingSphere.CreateFromPoints(convexSet);
			this.radius = sphere.Radius;
			this.points = convexSet;

            var mass = 1f;
            var inertia = Vector3.One;

            this.MassInverse = 1f / mass;
            this.InertiaInverse = Vector3.One / inertia;

            #region Assertions
            Debug.Assert(!float.IsNaN(this.InertiaInverse.X) && !float.IsInfinity(this.InertiaInverse.X));
            Debug.Assert(!float.IsNaN(this.InertiaInverse.Y) && !float.IsInfinity(this.InertiaInverse.Y));
            Debug.Assert(!float.IsNaN(this.InertiaInverse.Z) && !float.IsInfinity(this.InertiaInverse.Z));
            Debug.Assert(!float.IsNaN(this.MassInverse) && !float.IsInfinity(this.MassInverse));
            #endregion

            throw new global::System.NotImplementedException("I need to add in the mass-properties calculation code");
        }

        #region Methods
		/// <summary>Gets the radius of a bounding sphere which surrounds body</summary>
		public override float Radius
		{
			get { return this.radius; }
		}

        /// <summary>Finds the point (on the surface of this shape) most in the given direction</summary>
        /// <param name="direction">A direction vector that does not need to be normalized</param>
        /// <remarks>Coordinates are in the local frame</remarks>
        public override void Support(ref Vector3 direction, out Vector3 support)
        {
            #region Assertions
            Debug.Assert(!float.IsNaN(direction.X) && !float.IsInfinity(direction.X));
            Debug.Assert(!float.IsNaN(direction.Y) && !float.IsInfinity(direction.Y));
            Debug.Assert(!float.IsNaN(direction.Z) && !float.IsInfinity(direction.Z));
            Debug.Assert(direction.Length() > 0f);
            #endregion

            var normal = Vector3.Normalize(direction);

            float max = float.MinValue;
            foreach (var point in this.points)
            {
                if (Vector3.Dot(point, normal) > max)
                    support = point;
            }

            // This should never happen
            throw new InvalidOperationException();
        }
        #endregion
    }
}
