﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics.Geometries
{
    // .NET Fx
    using System.Diagnostics;

    // XNA
    using Microsoft.Xna.Framework;

    /// <summary>A sphere</summary>
    /// <remarks>http://en.wikipedia.org/wiki/Sphere</remarks>
    public sealed class Sphere : Body
    {
        #region Fields
        // Private variables
		private readonly float radius;
        #endregion

        public Sphere(Transform state, Material material, float radius)
            : base(state, material)
        {
            #region Assertions
            Debug.Assert(!float.IsNaN(radius));
            #endregion

            this.radius = radius;

            // The mass of a uniform sphere (given density)
            var mass = (4f / 3f) * MathHelper.Pi * 0.125f * material.Density;

            // Millington, Ian. "A.3 Common Shapes" Game Physics Engine Development - Elsevier, 2007. 432-433
            var inertia = Vector3.Zero;
            inertia.X = (2f / 5f) * mass * 0.25f;
            inertia.Y = (2f / 5f) * mass * 0.25f;
            inertia.Z = (2f / 5f) * mass * 0.25f;

            this.MassInverse = 1f / mass;
            this.InertiaInverse = Vector3.One / inertia;

            #region Assertions
            Debug.Assert(!float.IsNaN(this.InertiaInverse.X) && !float.IsInfinity(this.InertiaInverse.X));
            Debug.Assert(!float.IsNaN(this.InertiaInverse.Y) && !float.IsInfinity(this.InertiaInverse.Y));
            Debug.Assert(!float.IsNaN(this.InertiaInverse.Z) && !float.IsInfinity(this.InertiaInverse.Z));
            Debug.Assert(!float.IsNaN(this.MassInverse) && !float.IsInfinity(this.MassInverse));
            #endregion
        }

        #region Methods
		/// <summary>Gets the radius of a bounding sphere which surrounds body</summary>
		public override float Radius
		{
			get { return this.radius; }
		}

        /// <summary>Finds the point (on the surface of this shape) most in the given direction</summary>
        /// <param name="direction">A direction vector that does not need to be normalized</param>
        /// <remarks>Coordinates are in the local frame</remarks>
        public override void Support(ref Vector3 direction, out Vector3 support)
        {
            #region Assertions
            Debug.Assert(!float.IsNaN(direction.X) && !float.IsInfinity(direction.X));
            Debug.Assert(!float.IsNaN(direction.Y) && !float.IsInfinity(direction.Y));
            Debug.Assert(!float.IsNaN(direction.Z) && !float.IsInfinity(direction.Z));
            Debug.Assert(direction.Length() > 0f);
            #endregion

            // See http://www.win.tue.nl/~gino/solid/jgt98convex.pdf by Gino van den Bergen
            support = Vector3.Normalize(direction) * this.Radius;
        }
        #endregion
    }
}
