/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics
{
    // .NET Fx
    using System.Diagnostics;
    using System.Runtime.InteropServices;

    // XNA
    using Microsoft.Xna.Framework;

    /// <summary>An immutable structure that defines the basic properties of a body in
    /// relation to other bodies</summary>
    [StructLayout(LayoutKind.Auto)]
    public struct Material
    {
        #region Fields
        // Statics
        public static readonly Material Ice = new Material(0.25f, 0.05f, 0.01f);
        public static readonly Material Metal = new Material(1f, 0.9f, 0.05f);
        public static readonly Material Rubber = new Material(1f, 1f, 0.95f);
        public static readonly Material Wood = new Material(0.1f, 0.8f, 0.15f);

        /// <summary>Gets a measure of how tightly the matter within a body is packed together</summary>
        /// <remarks>A value greater than zero and less than infinity</remarks>
        public readonly float Density;

        /// <summary>Gets a measure of resistance between two moving bodies</summary>
        /// <remarks>The coefficient of friction (also known as the frictional coefficient) is a
        /// dimensionless scalar value which describes the ratio of the force of friction between
        /// two bodies and the force pressing them together. The coefficient of friction depends
        /// on the materials used; for example, ice on steel has a low coefficient of friction
        /// (the two materials slide past each other easily), while rubber on pavement has a high
        /// coefficient of friction (the materials do not slide past each other easily).
        /// Coefficients of friction range from near zero to greater than one  under good
        /// conditions, a tire on concrete may have a coefficient of friction of 1.7.</remarks>
        public readonly float Friction;

        // TODO: Static friction too??

        /// <summary>Get The coefficient of restitution of an object (a fractional value
        /// representing the ratio of velocities before and after an impact)</summary>
        /// <remarks>A value zero or greater and one or less</remarks>
        public readonly float Elasticity;
        #endregion

        public Material(float density, float friction, float elasticity)
        {
            #region Assertions
            Debug.Assert(!float.IsNaN(density) && !float.IsInfinity(density));
            Debug.Assert(!float.IsPositiveInfinity(density));
            Debug.Assert(!float.IsNaN(friction) && !float.IsInfinity(friction));
            Debug.Assert(!float.IsNaN(elasticity) && !float.IsInfinity(elasticity));
            Debug.Assert(density > 0f);
            Debug.Assert(friction >= 0f && friction <= 1f);
            Debug.Assert(elasticity >= 0f && elasticity <= 1f);
            #endregion

            // Set the values, but silently keep them within the proper ranges
            this.Density = MathHelper.Max(density, float.Epsilon);
            this.Friction = MathHelper.Clamp(friction, 0f, 1f);
            this.Elasticity = MathHelper.Clamp(elasticity, 0f, 1f);
        }
    } 
}