﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics.Pipeline
{
    // .NET Fx
    using System;
    using System.Collections.Generic;

    // XNA
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Content;

    // Local
    using JohnWells.GameDynamics.Constraints;
    using JohnWells.GameDynamics.Geometries;

    /// <summary>This type supports the GameDynamics infrastructure and is not intended to
    /// be used directly from your code</summary>
    internal static class Extensions
    {
        /// <summary>Reads a list of Body instances from the current stream</summary>
        public static IEnumerable<Body> ReadBodies(this ContentReader input)
        {
            // Count of bodies
            var count = input.ReadUInt32();
            if (count == 0)
                yield break;

            // Read bodies
            for (var index = 0; index < count; index++)
                switch (input.ReadResourceId())
                {
                    case ResourceId.Box:
                        yield return input.ReadBox();
                        break;

                    case ResourceId.ConvexHull:
                        yield return input.ReadConvexHull();
                        break;

                    case ResourceId.Sphere:
                        yield return input.ReadSphere();
                        break;

                    default:
                        throw new NotSupportedException();
                }
        }

        /// <summary>Reads a Box instance from the current stream</summary>
        public static Box ReadBox(this ContentReader input)
        {
            var state = input.RoadTransform();
            var material = input.ReadMaterial();
            var scale = input.ReadVector3();

            return new Box(state, material, scale);
        }

        /// <summary>Reads a list of IConstraint instances from the current stream</summary>
        /// <remarks>Requires the array of bodies to allow each constraint to reference particular instances</remarks>
        public static IEnumerable<IConstrained> ReadIConstraints(this ContentReader input, Body[] array)
        {
            // Count of constraints
            var count = input.ReadUInt32();
            if (count == 0)
                yield break;

            // Read constraints
            for (var index = 0; index < count; index++)
                switch (input.ReadResourceId())
                {
                    case ResourceId.PlaneConstraint:
                        yield return input.ReadPlaneConstraint(array);
                        break;

                    default:
                        throw new NotSupportedException();
                }
        }

        /// <summary>Reads a ConvexHull instance from the current stream</summary>
        public static ConvexHull ReadConvexHull(this ContentReader input)
        {
            var state = input.RoadTransform();
            var material = input.ReadMaterial();
            var points = input.ReadVector3List();

            return new ConvexHull(state, material, points);
        }

        /// <summary>Reads a Material value from the current stream</summary>
        public static Material ReadMaterial(this ContentReader input)
        {
            var density = input.ReadSingle();
            var friction = input.ReadSingle();
            var elasticity = input.ReadSingle();

            return new Material(density, friction, elasticity);
        }

        /// <summary>Reads a Plane value from the current stream</summary>
        public static Plane ReadPlane(this ContentReader input)
        {
            var a = input.ReadSingle();
            var b = input.ReadSingle();
            var c = input.ReadSingle();
            var d = input.ReadSingle();

            return new Plane(a, b, c, d);
        }

        /// <summary>Reads a PlaneConstraint instance from the current stream</summary>
        public static PlaneConstraint ReadPlaneConstraint(this ContentReader input, Body[] bodies)
        {
            var index = input.ReadInt32();
            var material = input.ReadMaterial();
            var plane = input.ReadPlane();

            return new PlaneConstraint(bodies[index], plane, material);
        }

        /// <summary>Reads a ResourceId value from the current stream</summary>
        public static ResourceId ReadResourceId(this ContentReader input)
        {
            return (ResourceId)input.ReadByte();
        }

        /// <summary>Reads a Sphere instance from the current stream</summary>
        public static Sphere ReadSphere(this ContentReader input)
        {
            var state = input.RoadTransform();
            var material = input.ReadMaterial();
            var radius = input.ReadSingle();

            return new Sphere(state, material, radius);
        }

        /// <summary>Reads a Transform value from the current stream</summary>
        public static Transform RoadTransform(this ContentReader input)
        {
            var translation = input.ReadVector3();
            var orientation = input.ReadQuaternion();

            return new Transform(translation, orientation);
        }

        /// <summary>Reads a list of Vector3 values from the current stream</summary>
        public static IEnumerable<Vector3> ReadVector3List(this ContentReader input)
        {
            // Count of vectors
            var count = input.ReadUInt32();
            if (count == 0)
                yield break;

            // Read vectors
            for (var index = 0; index < count; index++)
                yield return input.ReadVector3();
        }
    }
}
