﻿/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

#define INLINE_MATH

namespace JohnWells.GameDynamics
{
    // .NET Fx
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Runtime.InteropServices;

    // XNA
    using Microsoft.Xna.Framework;

    public abstract partial class Sap
    {
        /// <summary>Used by the FindPairs method</summary>
        /// <remarks>When possible and practical, specifically enumerating through this type
        /// (instead of IEnumerable) can prevent some GC costs associated with boxing</remarks>
        [StructLayout(LayoutKind.Auto)]
        public struct PairEnumerator : IEnumerable<Body>, IEnumerator<Body>
        {
            #region Fields
            // Private variables
            private readonly Sap instance;
            private readonly int version;
            private readonly Body collider;
            private int index;
            private Body current;
            #endregion

            /// <param name="segment">A segment of an array of delimiters, the beginning and end of which must be a
            /// pair of delimiters for and object which is being checked for interference</param>
            /// <param name="comparer">This is the comparer used to sort <param name="array" />, it is needed to
            /// make sure we don't yield bodies that we've already yielded</param>
            internal PairEnumerator(Sap instance, Body collider, int index)
            {
                #region Assertions
                Debug.Assert(instance != null);
                Debug.Assert(instance.bodies != null);
                Debug.Assert(instance.markers != null);
                Debug.Assert(instance.count <= instance.bodies.Length);
                Debug.Assert(instance.bodies.Length == instance.markers.Length);
                Debug.Assert(collider != null);
                Debug.Assert(index >= 0);
                #endregion

                this.instance = instance;
                this.version = instance.version;
                this.collider = collider;
                this.index = index;
                this.current = null;
            }

            #region Methods
            public Body Current
            {
                get { return this.current; }
            }

            public PairEnumerator GetEnumerator()
            {
                return this;
            }

            public bool MoveNext()
            {
                #region Assertions
                // Always make sure the instance hasn't been modified out from under us...
                Debug.Assert(this.instance.version == version, "Collection was modified; enumeration operation may not execute.");
                #endregion

                while (this.index < this.instance.count)
                {
					// TODO: Instrument
                    // index is incremented for next time..
                    this.current = this.instance.bodies[this.index++];

                    // If this body does not cover 'collider', we're done!
                    // (we can't pair with anything later...)
                    if (this.instance.comparer.IsDisjointed(this.collider, this.current))
                        return false;

#if !INLINE_MATH
                    // Skip non-intersecting bodies
					var currentBounds = new BoundingSphere(this.current.State.Translation, this.current.Radius);
                    if (!this.collider.Bounds.Intersects(currentBounds))
                        continue;
#else
					// Ericson, Christer. "4.3.1 Sphere Intersection" Real-time Collision Detection - Elsevier, 2005. 88
                    // Calculate the squared distance between centers
					var x = this.collider.State.Translation.X - this.current.State.Translation.X;
					var y = this.collider.State.Translation.Y - this.current.State.Translation.Y;
					var z = this.collider.State.Translation.Z - this.current.State.Translation.Z;
                    var distanceSqr = x * x + y * y + z * z;

                    // Spheres intersect if squared distance is less than squared sum of radii
                    var radiusSum = this.collider.Radius + this.current.Radius;
                    if (distanceSqr > radiusSum * radiusSum)
                        continue;
#endif

                    // We can't report a pair for 'collider'
                    if (object.ReferenceEquals(this.current, this.collider))
                        continue;

                    return true;
                }

                return false;
            } 
            #endregion

            #region IEnumerable<Body> Members
            IEnumerator<Body> IEnumerable<Body>.GetEnumerator()
            {
                return this;
            }

            IEnumerator IEnumerable.GetEnumerator()
            {
                return this;
            }
            #endregion

            #region IEnumerator<Body> Members
            Body IEnumerator<Body>.Current
            {
                get { return this.Current; }
            }

            void IDisposable.Dispose()
            {
            }

            object IEnumerator.Current
            {
                get { return this.Current; }
            }

            bool IEnumerator.MoveNext()
            {
                return this.MoveNext();
            }

            void IEnumerator.Reset()
            {
                // Sorry! I can't because I don't remember where I started! I could store it, but it's not
                // really a use-case that I see anyone needing. Why would you re-get a pair list anyway??
                throw new NotSupportedException();
            }
            #endregion
        }
    }
}