/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics
{
    // .NET Fx
    using System.Collections.Generic;
    
    // XNA
    using Microsoft.Xna.Framework;

    public abstract partial class Sap
    {
        /// <summary>This type supports the GameDynamics infrastructure and is not intended to be used directly from your code</summary>
        private interface IBodyComparer : IComparer<Body>
        {
            /// <summary>Gets a positive unit vector that describes the axis of comparison</summary>
            Vector3 Axis { get; }

            /// <summary>Returns true if the right hand side of body a is less than left hand side of body b</summary>
            /// <remarks>A true return value indicates that bodies a and b cannot be in contact, a false indicates that a and
            /// b may or may not be in contact</remarks>
            bool IsDisjointed(Body a, Body b);
        }

        /// <summary>This type supports the GameDynamics infrastructure and is not intended to be used directly from your code</summary>
        private sealed class XSort : IBodyComparer
        {
            #region IBodyComparer Members
            Vector3 IBodyComparer.Axis
            {
                get { return Vector3.UnitX; }
            }

            bool IBodyComparer.IsDisjointed(Body a, Body b)
            {
				return a.State.Translation.X + a.Radius < b.State.Translation.X - b.Radius;
            }
            #endregion

            #region IComparer<Body> Members
            int IComparer<Body>.Compare(Body a, Body b)
            {
				var an = a.State.Translation.X - a.Radius;
				var bn = b.State.Translation.X - b.Radius;

                // Compare start position on the X number line, and then IDs (Note: If
                // number line values are the same, IDs are most likely the same too)
                if (an < bn)
                    return -1;
                else if (an > bn)
                    return 1;
                else if (a.Id == b.Id)
                    return 0;
                else if (a.Id < b.Id)
                    return -1;
                else
                    return 1;
            }
            #endregion
        }

        /// <summary>This type supports the GameDynamics infrastructure and is not intended to be used directly from your code</summary>
        private sealed class YSort : IBodyComparer
        {
            #region IBodyComparer Members
            Vector3 IBodyComparer.Axis
            {
                get { return Vector3.UnitY; }
            }

            bool IBodyComparer.IsDisjointed(Body a, Body b)
            {
				return a.State.Translation.Y + a.Radius < b.State.Translation.Y - b.Radius;
            }
            #endregion

            #region IComparer<Body> Members
            int IComparer<Body>.Compare(Body a, Body b)
            {
				var an = a.State.Translation.Y - a.Radius;
				var bn = b.State.Translation.Y - b.Radius;

                // Compare start position on the Y number line, and then IDs (Note: If
                // number line values are the same, IDs are most likely the same too)
                if (an < bn)
                    return -1;
                else if (an > bn)
                    return 1;
                else if (a.Id == b.Id)
                    return 0;
                else if (a.Id < b.Id)
                    return -1;
                else
                    return 1;
            }
            #endregion
        }

        /// <summary>This type supports the GameDynamics infrastructure and is not intended to be used directly from your code</summary>
        private sealed class ZSort : IBodyComparer
        {
            #region IBodyComparer Members
            Vector3 IBodyComparer.Axis
            {
                get { return Vector3.UnitZ; }
            }

            bool IBodyComparer.IsDisjointed(Body a, Body b)
            {
				return a.State.Translation.Z + a.Radius < b.State.Translation.Z - b.Radius;
            }
            #endregion

            #region IComparer<Body> Members
            int IComparer<Body>.Compare(Body a, Body b)
            {
				var an = a.State.Translation.Z - a.Radius;
				var bn = b.State.Translation.Z - b.Radius;

                // Compare start position on the Z number line, and then IDs (Note: If
                // number line values are the same, IDs are most likely the same too)
                if (an < bn)
                    return -1;
                else if (an > bn)
                    return 1;
                else if (a.Id == b.Id)
                    return 0;
                else if (a.Id < b.Id)
                    return -1;
                else
                    return 1;
            }
            #endregion
        }
    }
}
