/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameDynamics
{
    // .NET Fx
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Diagnostics;

    // XNA
    using Microsoft.Xna.Framework;

    /// <summary>My take on an array-based sort-and-sweep broadphase</summary>
    public abstract partial class Sap : IBroadphase
    {
        #region Fields
        // Constants
        private const int DefaultCapacity = 4;
        private const int ComparisonFrequency = 25;

        // Private variables
        private Body[] bodies;
        private bool[] markers;
        private int version, count;
        private IBodyComparer comparer, nextComparer;
        private readonly IBodyComparer x, y, z;
        #endregion

        protected Sap()
        {
            this.count = this.version = 0;
            this.bodies = new Body[Sap.DefaultCapacity];
            this.markers = new bool[Sap.DefaultCapacity];

            // The body comparers are used to sort our bodies based on the axis they're least bunched up
            // on - so searching through them is as quick as possible - start arbitralily with x
            this.x = new XSort();
            this.y = new YSort();
            this.z = new ZSort();
            this.comparer = this.nextComparer = this.x;
        }

        #region Methods
        /// <summary>Adds the elements of the specified collection to the broadphase</summary>
        public void AddRange(IEnumerable<Body> collection)
        {
            #region Assertions
            Debug.Assert(collection != null);
            #endregion

            // Call our regular Add method
            foreach (var item in collection)
                this.Add(item);
        }

        /// <summary>A manual search that will work even when the bodies array is unsorted - use in place of BinarySearch as needed</summary>
        private int IndexOf(Body item)
        {
            // Step through the entire bodies array looking for the index of this body
            if (this.count > 0)
                for (var index = 0; index < this.count; index++)
                {
                    if (object.ReferenceEquals(this.bodies[index], item))
                        return index;
                }

            // Not found
            return -1;
        }

        /// <summary>Finds all bodies contained by this collection whose bounding volumes overlap the given body</summary>
        public PairEnumerator FindPairs(Body item)
        {
            #region Assertions
            Debug.Assert(item != null);
            #endregion

            // Find the index for this body
            var index = Array.BinarySearch(this.bodies, 0, this.count, item, this.comparer);

            // We should always get an index - Find out why we failed
            if (index < 0)
            {
                if (this.IndexOf(item) < 0)
                    // This body just isn't here - I know, I know... But I looked through it manually!
                    throw new InvalidOperationException("The given body does not exist in this collection");
                else
                    // The collection has become unsorted due to either misuse of the API, or a body
                    // has exploded into an area of the number line that fails floating point comparison
                    throw new InvalidOperationException("This collection is not sorted");
            }

            // Rewind from 'index' until we hit the previous marker or the beginning of the markers array
            while (index > 0)
            {
                index--;
                if (this.markers[index] == true)
                    break;
            }

            // Create the enumerator - index is the 'start of search' index for this body
            return new PairEnumerator(this, item, index);
        }

        /// <summary>Finds all bodies contained by this collection</summary>
        public Enumerator GetEnumerator()
        {
            return new Enumerator(this);
        }

        /// <summary>Provides a spatial query that returns bodies within the given area</summary>
        public QueryEnumerator Query(BoundingBox bounds, ContainmentType mode)
        {
            return new QueryEnumerator(this, bounds, mode);
        }

        /// <summary>Sorts the underlying lists that this collection relies upon</summary>
        /// <remarks>This method must be called before the FindInterference or Query methods are called</remarks>
        public void Sort()
        {
            // Filter: Skip this logic when we have no bodies
            if (this.count == 0)
                return;

            // Sort the bodies using the latest comparer (as set during the last sort...)
            this.comparer = this.nextComparer;
            Array.Sort(this.bodies, 0, this.count, this.comparer);
            this.version++;

            var axis = this.comparer.Axis;
            var markAt = float.MinValue;

            // Set the marker flags that we use to find empty spots in the bodies array
            for (var index = 0; index < this.count; index++)
            {
                // Find the center value of this body on the current axial number line
                var body = this.bodies[index];
                var center = Vector3.Dot(body.State.Translation, axis);

                // Set the marker to true if this body's min is after our next 'Mark At' position
                // (Otherwise we set this marker to false to indicate this space on the line is taken)
                this.markers[index] = (center - body.Radius > markAt);

                // Update the 'Mark At' position to be the further of either what we currently have
                // or this body's max position
				markAt = Math.Max(markAt, center + body.Radius);
            }

            // TODO: Is this worth it or should I do this above
            // Occasionally we determine how the bodies are grouped and see if we should use a different
            // comparer - but we only do this when enough time has passed that it's worth it...
            if (this.version % Sap.ComparisonFrequency == 0)
            {
                var sum = Vector3.Zero;
                var squares = Vector3.Zero;

                foreach (var item in this.GetEnumerator())
                {
                    sum += item.State.Translation;
					squares += item.State.Translation * item.State.Translation;
                }

                var variance = (squares - sum * (sum / this.count)) / this.count;

                // Set the comparer we'll use next time to whichever axis has the most variance
                this.nextComparer = this.z;
                if (variance.X > variance.Y && variance.X > variance.Z)
                    this.nextComparer = this.x;
                else if (variance.Y > variance.Z)
                    this.nextComparer = this.y;
            }
        }
        #endregion

        #region IBroadphase Members
        IEnumerable<Body> IBroadphase.FindPairs(Body body)
        {
            return this.FindPairs(body);
        }

        IEnumerable<Body> IBroadphase.Query(BoundingBox bounds, ContainmentType mode)
        {
            return this.Query(bounds, mode);
        }
        #endregion

        #region ICollection<Body> Members
        /// <summary>Adds a body to the broadphase</summary>
        public void Add(Body item)
        {
            #region Assertions
            Debug.Assert(item != null);
            Debug.Assert(this.IndexOf(item) < 0);
            #endregion

            var index = this.count;

            // Autogrow the arrays in large steps, only when needed
            this.count++;
            if (this.count > this.bodies.Length)
            {
                Array.Resize(ref this.bodies, this.bodies.Length * 2);
                Array.Resize(ref this.markers, this.bodies.Length);
            }

            // Add this body to the array
            this.bodies[index] = item;
            this.markers[index] = false;
            this.version++;
        }

        void ICollection<Body>.Clear()
        {
            this.count = 0;
            this.bodies.Initialize();
            this.markers.Initialize();
            this.version++;
        }

        /// <summary>Determines whether a body is in the broadphase</summary>
        bool ICollection<Body>.Contains(Body item)
        {
            #region Assertions
            Debug.Assert(item != null);
            #endregion

            // Try to find the index this body
            var index = Array.BinarySearch(this.bodies, 0, this.count, item, this.comparer);

            // If the array is not sorted; look harder...
            if (index < 0)
                index = this.IndexOf(item);

            // If BinarySearch/IndexOf returns -1 or less we didn't find it
            return index >= 0;
        }

        void ICollection<Body>.CopyTo(Body[] array, int arrayIndex)
        {
            Array.Copy(this.bodies, 0, array, arrayIndex, 0);
        }

        int ICollection<Body>.Count
        {
            get { return this.count; }
        }

        bool ICollection<Body>.IsReadOnly
        {
            get { return false; }
        }

        /// <summary>Removes a body from the broadphase</summary>
        public bool Remove(Body item)
        {
            #region Assertions
            Debug.Assert(item != null);
            #endregion

            // Find the index of this body in our sorted array
            var index = Array.BinarySearch(this.bodies, 0, this.count, item, this.comparer);

            // If the array is not sorted; look harder...
            if (index < 0)
                index = this.IndexOf(item);

            // If we couldn't find this body just return false
            if (index < 0)
                return false;

            this.count--;

            // Shift all elements left after this item
            if (index < this.bodies.Length - 1)
            {
                Array.Copy(this.bodies, index + 1, this.bodies, index, this.count - index);
                Array.Copy(this.markers, index + 1, this.markers, index, this.count - index);
            }

            // Reset the last elements (which is at this point a copy of the second to last element...)
            this.bodies[this.count] = null;
            this.markers[this.count] = false;
            this.version++;

            return true;
        }
        #endregion

        #region IEnumerable<Body> Members
        IEnumerator<Body> IEnumerable<Body>.GetEnumerator()
        {
            return this.GetEnumerator();
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }
        #endregion
    }
}
