/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

#define INLINE_MATH

namespace JohnWells.GameDynamics
{
    // .NET Fx
    using System;
    using System.Diagnostics;
    using System.Runtime.InteropServices;

    // XNA
    using Microsoft.Xna.Framework;

    /// <summary>An immutable structure defining the translation and orientation of a body</summary>
    [StructLayout(LayoutKind.Auto)]
    public struct Transform
    {
        #region Fields
        /// <summary>A state whose transformation does not alter the input</summary>
        public readonly static Transform Identity = new Transform(Vector3.Zero, Quaternion.Identity);

        /// <summary>Displacement from the origin</summary>
        public readonly Vector3 Translation;
        
        /// <summary>Unit length quaternion</summary>
        public readonly Quaternion Orientation;
        #endregion

        public Transform(Vector3 translation, Quaternion orientation)
            : this(ref translation, ref orientation)
        {
        }

        internal Transform(ref Vector3 translation, ref Quaternion orientation)
        {
            #region Assertions
            Debug.Assert(!float.IsNaN(translation.X) && !float.IsInfinity(translation.X));
            Debug.Assert(!float.IsNaN(translation.Y) && !float.IsInfinity(translation.Y));
            Debug.Assert(!float.IsNaN(translation.Z) && !float.IsInfinity(translation.Z));
            Debug.Assert(!float.IsNaN(orientation.W) && !float.IsInfinity(orientation.W));
            Debug.Assert(!float.IsNaN(orientation.X) && !float.IsInfinity(orientation.X));
            Debug.Assert(!float.IsNaN(orientation.Y) && !float.IsInfinity(orientation.Y));
            Debug.Assert(!float.IsNaN(orientation.Z) && !float.IsInfinity(orientation.Z));
            Debug.Assert(orientation.Length() > 0.99f);
            Debug.Assert(orientation.Length() < 1.01f);
            #endregion

            this.Translation = translation;
            this.Orientation = orientation;
        }

        #region Methods
        /// <summary>Useful for using a state instance as if it were a transform matrix</summary>
        public static implicit operator Matrix(Transform value)
        {
#if !INLINE_MATH
            return Matrix.CreateFromQuaternion(value.Orientation) * Matrix.CreateTranslation(value.Translation);
#else
            var xx = value.Orientation.X * value.Orientation.X;
            var yy = value.Orientation.Y * value.Orientation.Y;
            var zz = value.Orientation.Z * value.Orientation.Z;
            var xy = value.Orientation.X * value.Orientation.Y;
            var zw = value.Orientation.Z * value.Orientation.W;
            var zx = value.Orientation.Z * value.Orientation.X;
            var yw = value.Orientation.Y * value.Orientation.W;
            var yz = value.Orientation.Y * value.Orientation.Z;
            var xw = value.Orientation.X * value.Orientation.W;

            return new Matrix(1f - 2f * (yy + zz),
                              2f * (xy + zw),
                              2f * (zx - yw),
                              0f,
                              2f * (xy - zw),
                              1f - 2f * (zz + xx),
                              2f * (yz + xw),
                              0f,
                              2f * (zx + yw),
                              2f * (yz - xw),
                              1f - 2f * (yy + xx),
                              0f,
                              value.Translation.X,
                              value.Translation.Y,
                              value.Translation.Z,
                              1f);
#endif
        }
        #endregion
    }
}
