/* Copyright (c) 2008 John Wells - http://www.codeplex.com/Johnnylightbulb/

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use,
copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */

namespace JohnWells.GameFx
{
    // .NET Fx
    using System;

    // XNA
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Graphics;
    using Microsoft.Xna.Framework.Content;

    /// <summary>A Screen is a single layer that has update and draw logic, and which can be combined with other
    /// Screen instances to create an encapsulated, state-based game system</summary>
    public abstract class GameScreen : DrawableGameComponent
    {
        #region Fields
        // Private variables
        private readonly string contentFolder;
        private ContentManager content;
        private SpriteBatch spriteBatch;
        private GameScreen child;
        private TransitionState state;
        private TimeSpan elapsed;
        private float amount;
        #endregion

        protected GameScreen(Game game)
            : base(game)
        {
        }

        protected GameScreen(Game game, string contentFolder)
            : base(game)
        {
            // This is the folder all content will be rooted it - It can save sanity to set it if your
            // content folder is really deep or changes parent folder names (For example, many levels with
            // different graphics for the same named asset)
            this.contentFolder = contentFolder;
        }

        #region Properties
        public GameScreen ChildScreen
        {
            get { return this.child; }
        }

        /// <summary>Indicates how long the screen takes to transition on when it is activated</summary>
        /// <remarks>Default is zero seconds (instant)</remarks>
        public virtual TimeSpan TransitionOnTime
        {
            get { return TimeSpan.Zero; }
        }

        /// <summary>Indicates how long the screen takes to transition off when it is deactivated</summary>
        /// <remarks>Default is zero seconds (instant)</remarks>
        public virtual TimeSpan TransitionOffTime
        {
            get { return TimeSpan.Zero; }
        }

        /// <summary>Gets the current 'transitioned-ness' of this screen. This starts at 0 when the screen is transitioning
        /// on, builds to and stays at 1 while the screen is active, and drops to 0 again when the screen transitions off</summary>
        public float Amount
        {
            get
            {
                if (this.state == TransitionState.Steady)
                    // Steady is always 1.0
                    return 1f;
                else
                    // Rising/Falling are in the range of [0,1]
                    return MathHelper.Clamp(this.amount, 0f, 1f);
            }
        }

        /// <summary>Gets the current transition state</summary>
        public TransitionState State
        {
            get { return this.state; }
        }

        /// <summary>A local content manager specific to this instance</summary>
        protected ContentManager Content
        {
            get { return this.content; }
        }

        /// <summary>A local sprite batch specific to this instance</summary>
        protected SpriteBatch SpriteBatch
        {
            get { return this.spriteBatch; }
        }
        #endregion

        #region Methods
        protected override void LoadContent()
        {
            base.LoadContent();

            // Load our local content manager
            if (!string.IsNullOrEmpty(this.contentFolder))
                this.content = new ContentManager(this.Game.Services, this.contentFolder);
            else
                this.content = new ContentManager(this.Game.Services);

            // Create the sprite batch object
            this.spriteBatch = new SpriteBatch(this.Game.GraphicsDevice);
        }

        protected override void UnloadContent()
        {
            base.UnloadContent();

            // Unload our local content manager
            this.content.Unload();
        } 

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);

            if (!disposing)
                return;

            // Dispose of the content manager
            if (this.content != null)
            {
                this.content.Dispose();
                this.content = null;
            }

            // Dispose of the sprite batch
            if (this.spriteBatch != null)
            {
                this.spriteBatch.Dispose();
                this.spriteBatch = null;
            }

            GC.SuppressFinalize(this);
        }

        public override void Initialize()
        {
            base.Initialize();

            // Set the initial state (Off and ready to rise)
            this.state = TransitionState.Rising;
            this.elapsed = TimeSpan.Zero;
            this.amount = 0f;
        }

        /// <summary>Allows the screen to run logic, such as updating the transition position - Unlike HandleInput, this
        /// method is called regardless of whether the screen is active, hidden, or in the middle of a transition</summary>
        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);

            // Update the 'elapsed' and 'amount' values and transition state
            switch (this.state)
            {
                case TransitionState.Rising:
                    this.elapsed = this.elapsed.Add(gameTime.ElapsedGameTime);
                    this.amount = (float)this.elapsed.TotalSeconds / (float)this.TransitionOnTime.TotalSeconds;

                    // Are we at the steady state yet?
                    if (this.amount >= 1f)
                        this.state = TransitionState.Steady;
                    break;

                case TransitionState.Falling:
                    this.elapsed = this.elapsed.Add(gameTime.ElapsedGameTime);
                    this.amount = 1f - (float)this.elapsed.TotalSeconds / (float)this.TransitionOffTime.TotalSeconds;

                    // Are we at the disabled state yet?
                    if (this.amount <= 0f)
                        this.Enabled = false;
                    break;

                default:
                    this.elapsed = TimeSpan.Zero;
                    this.amount = 1f;
                    break;
            }

            // Remove the child if it has been disabled
            if (this.child != null && !this.child.Enabled)
            {
                // We only remove the reference - We don't dispose because we didn't call new... fyi
                this.Game.Components.Remove(this.child);
                this.child = null;
            }
        }

        public void Show(GameScreen screen)
        {
            // Intialize the screen and make it our child
            screen.Enabled = true;
            screen.Initialize();
            this.child = screen;

            // Add it to our game's components collection
            // (This doesn't add the screen if it's still being removed...)
            if (!this.Game.Components.Contains(screen))
                this.Game.Components.Add(screen);
        }

        /// <summary>Tells the screen to go away with a gradual transition off</summary>
        public void Close()
        {
            // Start the transistion timer over again
            this.elapsed = TimeSpan.Zero;
            this.state = TransitionState.Falling;

            // If the screen has a zero transition time, disable it immediately
            if (this.TransitionOffTime <= TimeSpan.Zero)
            {
                this.Game.Components.Remove(this);
                this.Enabled = false;
            }
        }
        #endregion
    }
}